using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace Bespoke.Common
{
    /// <summary>
    /// 
    /// </summary>
    public delegate void VoidHandler();

	/// <summary>
	/// 
	/// </summary>
	/// <param name="progressBar"></param>
	/// <param name="value"></param>
	public delegate void UpdateProgressValueHandler(ProgressBar progressBar, int value);

	/// <summary>
	/// 
	/// </summary>
	/// <param name="label"></param>
	/// <param name="text"></param>
	public delegate void UpdateLabelHandler(Label label, string text);

	/// <summary>
	/// 
	/// </summary>
	/// <param name="textBox"></param>
	/// <param name="text"></param>
	public delegate void UpdateTextBoxHandler(TextBox textBox, string text);

	/// <summary>
	/// 
	/// </summary>
	/// <param name="button"></param>
	/// <param name="text"></param>
	public delegate void UpdateButtonTextHandler(Button button, string text);

	/// <summary>
	/// 
	/// </summary>
	/// <param name="pictureBox"></param>
	/// <param name="image"></param>
	public delegate void UpdatePictureBoxImageHandler(PictureBox pictureBox, Image image);

	/// <summary>
	/// 
	/// </summary>
	public static class Library
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sourceDirectory"></param>
		/// <param name="destinationDirectory"></param>
		public static void CopyDirectory(string sourceDirectory, string destinationDirectory)
		{
			DirectoryInfo sourceDirectoryInfo = new DirectoryInfo(sourceDirectory);
			if (sourceDirectoryInfo.Exists == false)
			{
				throw new DirectoryNotFoundException(sourceDirectoryInfo.FullName);
			}

			DirectoryInfo destinationDirectoryInfo = new DirectoryInfo(destinationDirectory);
			if (destinationDirectoryInfo.Exists == false)
			{
				destinationDirectoryInfo.Create();
			}

			foreach (DirectoryInfo subDirectoryInfo in sourceDirectoryInfo.GetDirectories())
			{
				CopyDirectory(subDirectoryInfo.FullName, destinationDirectoryInfo.FullName + Path.DirectorySeparatorChar + subDirectoryInfo.Name);
			}

			foreach (FileInfo fileInfo in sourceDirectoryInfo.GetFiles())
			{
				fileInfo.CopyTo(destinationDirectoryInfo.FullName + Path.DirectorySeparatorChar + fileInfo.Name, true);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value1"></param>
		/// <param name="value2"></param>
		/// <param name="proxmityThreshold"></param>
		/// <returns></returns>
        public static bool ValuesInProximity(int value1, int value2, int proxmityThreshold)
        {
            return Math.Abs(value1 - value2) <= proxmityThreshold;
        }

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value1"></param>
		/// <param name="value2"></param>
		/// <param name="proxmityThreshold"></param>
		/// <returns></returns>
        public static bool ValuesInProximity(double value1, double value2, double proxmityThreshold)
        {
            return Math.Abs(value1 - value2) <= proxmityThreshold;
        }

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value1"></param>
		/// <param name="value2"></param>
		/// <param name="proxmityThreshold"></param>
		/// <returns></returns>
        public static bool ValuesInProximity(DateTime value1, DateTime value2, TimeSpan proxmityThreshold)
        {
            TimeSpan difference;

            if (value1 > value2)
            {
                difference = value1.Subtract(value2);
            }
            else
            {
                difference = value2.Subtract(value1);
            }

            return difference <= proxmityThreshold;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static bool IsNumeric(string value)
        {
            int result;
            return IsNumeric(value, out result);
        }

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value"></param>
		/// <param name="result"></param>
		/// <returns></returns>
        public static bool IsNumeric(string value, out int result)
        {
            if (int.TryParse(value, out result))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
	}
}
