﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class PublicationSourceTypeManagerTest : TestFixtureBase
    {
        public static PublicationSourceTypeRecord CreatePublicationSourceType()
        {
            PublicationSourceTypeRecord publicationSourceType = new PublicationSourceTypeRecord(SqlDataProvider.DefaultKey, "Test PublicationSourceType");
            PublicationSourceTypeManager.AddPublicationSourceType(publicationSourceType);
            Trace.WriteLine("CreatePublicationSourceType(): PublicationSourceType added.");

            return publicationSourceType;
        }

        public static void DeletePublicationSourceType(PublicationSourceTypeRecord publicationSourceType)
        {
            PublicationSourceTypeManager.DeletePublicationSourceType(publicationSourceType);
            Trace.WriteLine("DeletePublicationSourceType(): PublicationSourceType deleted.");

            PublicationSourceTypeRecord foundPublicationSourceType;
            bool found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByKey(publicationSourceType.Key, out foundPublicationSourceType);
            Assert.IsFalse(found, "DeletePublicationSourceType(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mPublicationSourceType = CreatePublicationSourceType();
        }

        [TearDown]
        public void TearDown()
        {
            DeletePublicationSourceType(mPublicationSourceType);
        }

        [Test]
        public void AddPublicationSourceType()
        {
            Trace.WriteLine("AddPublicationSourceType(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeletePublicationSourceType()
        {
            PublicationSourceTypeRecord publicationSourceType = new PublicationSourceTypeRecord(SqlDataProvider.DefaultKey, "Delete PublicationSourceType");
            PublicationSourceTypeManager.AddPublicationSourceType(publicationSourceType);
            Trace.WriteLine("DeletePublicationSourceType(): PublicationSourceType added.");

            PublicationSourceTypeRecord foundPublicationSourceType;
            bool found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByKey(publicationSourceType.Key, out foundPublicationSourceType);
            Assert.IsTrue(found, "DeletePublicationSourceType():Pre-delete operation(s) failed.");

            DeletePublicationSourceType(publicationSourceType);
        }

        [Test]
        public void LoadPublicationSourceType()
        {
            PublicationSourceTypeRecordList publicationSourceTypeList = PublicationSourceTypeManager.LoadPublicationSourceType();
            Assert.IsTrue(publicationSourceTypeList.Count > 0);

            Trace.WriteLine("LoadPublicationSourceType(): " + publicationSourceTypeList.Count + " PublicationSourceTypes loaded.");
        }

        [Test]
        public void RetrievePublicationSourceTypeByKey()
        {
            PublicationSourceTypeRecord foundPublicationSourceType;
            bool found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByKey(mPublicationSourceType.Key, out foundPublicationSourceType);
            Assert.IsTrue(found, "RetrievePublicationSourceTypeByKey(): PublicationSourceType not found.");
            Assert.IsTrue(mPublicationSourceType == foundPublicationSourceType);

            Trace.WriteLine("RetrievePublicationSourceTypeByKey(): PublicationSourceType retrieved.");
        }

        [Test]
        public void RetrievePublicationSourceTypeByName()
        {
            PublicationSourceTypeRecord foundPublicationSourceType;
            bool found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByName(mPublicationSourceType.Name, out foundPublicationSourceType);
            Assert.IsTrue(found, "RetrievePublicationSourceTypeByName(): PublicationSourceType not found.");
            Assert.IsTrue(mPublicationSourceType == foundPublicationSourceType);

            Trace.WriteLine("RetrievePublicationSourceTypeByName(): PublicationSourceType retrieved.");
        }

        [Test]
        public void UpdatePublicationSourceType()
        {
            PublicationSourceTypeRecord publicationSourceType = new PublicationSourceTypeRecord(SqlDataProvider.DefaultKey, "Update PublicationSourceType");
            PublicationSourceTypeManager.AddPublicationSourceType(publicationSourceType);
            Trace.WriteLine("UpdatePublicationSourceType(): PublicationSourceType added.");

            PublicationSourceTypeRecord foundPublicationSourceType;
            bool found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByKey(publicationSourceType.Key, out foundPublicationSourceType);
            Assert.IsTrue(found, "UpdatePublicationSourceType(): Pre-update operation(s) failed.");

            publicationSourceType.Name = "Updated Name";
            PublicationSourceTypeManager.UpdatePublicationSourceType(publicationSourceType);

            found = PublicationSourceTypeManager.RetrievePublicationSourceTypeByKey(publicationSourceType.Key, out foundPublicationSourceType);
            Assert.IsTrue(found, "UpdatePublicationSourceType(): Post-update operation(s) failed.");
            Assert.AreEqual(publicationSourceType.Name, foundPublicationSourceType.Name);

            DeletePublicationSourceType(publicationSourceType);
        }

        private PublicationSourceTypeRecord mPublicationSourceType;
    }
}
