﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class RepositoryGroupLinkManagerTest : TestFixtureBase
    {
        public static RepositoryGroupLinkRecord CreateRepositoryGroupLink(RepositoryRecord repository, GroupRecord group, int roleKey)
        {
            RepositoryGroupLinkRecord rgl = new RepositoryGroupLinkRecord(repository.Key, group.Key, roleKey);
            RepositoryGroupLinkManager.AddRepositoryGroupLink(rgl);
            Trace.WriteLine("CreateRepositoryGroupLink(): RepositoryGroupLink added.");

            return rgl;
        }

        public static void DeleteRepositoryGroupLink(RepositoryGroupLinkRecord rgl)
        {
            RepositoryGroupLinkManager.DeleteRepositoryGroupLink(rgl);
            Trace.WriteLine("DeleteRepositoryGroupLink(): RepositoryGroupLink deleted.");

            RepositoryGroupLinkRecord foundRepositoryGroupLink;
            bool found = RepositoryGroupLinkManager.RetrieveRepositoryGroupLinkByKey(rgl.RepositoryKey, rgl.GroupKey, out foundRepositoryGroupLink);
            Assert.IsFalse(found, "DeleteRepositoryGroupLink(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mRepository = RepositoryManagerTest.CreateRepository(mUser);
            mGroup = GroupManagerTest.CreateGroup(mUser);
            mRepositoryGroupLink = CreateRepositoryGroupLink(mRepository, mGroup, mDatabase.Roles[Role.User].Key);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteRepositoryGroupLink(mRepositoryGroupLink);
            RepositoryManagerTest.DeleteRepository(mRepository);
            GroupManagerTest.DeleteGroup(mGroup);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddRepositoryGroupLink()
        {
            Trace.WriteLine("AddRepositoryGroupLink(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteRepositoryGroupLink()
        {
            Trace.WriteLine("AddRepositoryGroupLink(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void LoadRepositoryGroupLinkByRepositoryKey()
        {
            RepositoryGroupLinkRecordList rglList = RepositoryGroupLinkManager.LoadRepositoryGroupLinkByRepositoryKey(mRepository.Key);
            Assert.IsTrue(rglList.Count > 0);

            Trace.WriteLine("LoadRepositoryGroupLinkByRepositoryKey(): " + rglList.Count + " Repository-Group Link Records loaded.");
        }

        [Test]
        public void LoadRepositoryGroupLinkByGroupKey()
        {
            RepositoryGroupLinkRecordList rglList = RepositoryGroupLinkManager.LoadRepositoryGroupLinkByGroupKey(mGroup.Key);
            Assert.IsTrue(rglList.Count > 0);

            Trace.WriteLine("LoadRepositoryGroupLinkByGroupKey(): " + rglList.Count + " Repository-Group Link Records loaded.");
        }

        [Test]
        public void RetrieveRepositoryGroupLinkByKey()
        {
            RepositoryGroupLinkRecord foundRepositoryGroupLink;
            bool found = RepositoryGroupLinkManager.RetrieveRepositoryGroupLinkByKey(mRepository.Key, mGroup.Key, out foundRepositoryGroupLink);
            Assert.IsTrue(found, "RetrieveRepositoryGroupLinkByKey(): Repository-Group Link Record not found.");
            Assert.IsTrue(mRepositoryGroupLink == foundRepositoryGroupLink);

            Trace.WriteLine("RetrieveRepositoryGroupLinkByKey(): Repository-Group Link Record retrieved.");
        }

        [Test]
        public void UpdateRepositoryGroupLink()
        {
            mRepositoryGroupLink.RoleKey = mDatabase.Roles[Role.Administrator].Key;
            RepositoryGroupLinkManager.UpdateRepositoryGroupLink(mRepositoryGroupLink);

            RepositoryGroupLinkRecord foundRepositoryGroupLink;
            bool found = RepositoryGroupLinkManager.RetrieveRepositoryGroupLinkByKey(mRepositoryGroupLink.RepositoryKey, mRepositoryGroupLink.GroupKey, out foundRepositoryGroupLink);
            Assert.IsTrue(found, "UpdateRepositoryGroupLink(): Post-update operation(s) failed.");
            Assert.AreEqual(mRepositoryGroupLink.RoleKey, foundRepositoryGroupLink.RoleKey);
        }

        private RepositoryRecord mRepository;
        private UserRecord mUser;
        private GroupRecord mGroup;
        private RepositoryGroupLinkRecord mRepositoryGroupLink;
    }
}
