﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class RepositoryUserLinkManagerTest : TestFixtureBase
    {
        public static RepositoryUserLinkRecord CreateRepositoryUserLink(RepositoryRecord repository, UserRecord user, int roleKey)
        {
            RepositoryUserLinkRecord rul = new RepositoryUserLinkRecord(repository.Key, user.Key, roleKey);
            RepositoryUserLinkManager.AddRepositoryUserLink(rul);
            Trace.WriteLine("CreateRepositoryUserLink(): RepositoryUserLink added.");

            return rul;
        }

        public static void DeleteRepositoryUserLink(RepositoryUserLinkRecord rul)
        {
            RepositoryUserLinkManager.DeleteRepositoryUserLink(rul);
            Trace.WriteLine("DeleteRepositoryUserLink(): RepositoryUserLink deleted.");

            RepositoryUserLinkRecord foundRepositoryUserLink;
            bool found = RepositoryUserLinkManager.RetrieveRepositoryUserLinkByKey(rul.RepositoryKey, rul.UserKey, out foundRepositoryUserLink);
            Assert.IsFalse(found, "DeleteRepositoryUserLink(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mRepository = RepositoryManagerTest.CreateRepository(mUser);
            mRepositoryUserLink = CreateRepositoryUserLink(mRepository, mUser, mDatabase.Roles[Role.User].Key);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteRepositoryUserLink(mRepositoryUserLink);
            RepositoryManagerTest.DeleteRepository(mRepository);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddRepositoryUserLink()
        {
            Trace.WriteLine("AddRepositoryUserLink(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteRepositoryUserLink()
        {
            Trace.WriteLine("AddRepositoryUserLink(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void LoadRepositoryUserLinkByRepositoryKey()
        {
            RepositoryUserLinkRecordList rulList = RepositoryUserLinkManager.LoadRepositoryUserLinkByRepositoryKey(mRepository.Key);
            Assert.IsTrue(rulList.Count > 0);

            Trace.WriteLine("LoadRepositoryUserLinkByRepositoryKey(): " + rulList.Count + " Repository-User Link Records loaded.");
        }

        [Test]
        public void LoadRepositoryUserLinkByUserKey()
        {
            RepositoryUserLinkRecordList rulList = RepositoryUserLinkManager.LoadRepositoryUserLinkByUserKey(mUser.Key);
            Assert.IsTrue(rulList.Count > 0);

            Trace.WriteLine("LoadRepositoryUserLinkByUserKey(): " + rulList.Count + " Repository-User Link Records loaded.");
        }

        [Test]
        public void RetrieveRepositoryUserLinkByKey()
        {
            RepositoryUserLinkRecord foundRepositoryUserLink;
            bool found = RepositoryUserLinkManager.RetrieveRepositoryUserLinkByKey(mRepository.Key, mUser.Key, out foundRepositoryUserLink);
            Assert.IsTrue(found, "RetrieveRepositoryUserLinkByKey(): Repository-User Link Record not found.");
            Assert.IsTrue(mRepositoryUserLink == foundRepositoryUserLink);

            Trace.WriteLine("RetrieveRepositoryUserLinkByKey(): Repository-User Link Record retrieved.");
        }

        [Test]
        public void UpdateRepositoryUserLink()
        {
            mRepositoryUserLink.RoleKey = mDatabase.Roles[Role.Administrator].Key;
            RepositoryUserLinkManager.UpdateRepositoryUserLink(mRepositoryUserLink);

            RepositoryUserLinkRecord foundRepositoryUserLink;
            bool found = RepositoryUserLinkManager.RetrieveRepositoryUserLinkByKey(mRepositoryUserLink.RepositoryKey, mRepositoryUserLink.UserKey, out foundRepositoryUserLink);
            Assert.IsTrue(found, "UpdateRepositoryUserLink(): Post-update operation(s) failed.");
            Assert.AreEqual(mRepositoryUserLink.RoleKey, foundRepositoryUserLink.RoleKey);
        }

        private RepositoryRecord mRepository;
        private UserRecord mUser;
        private RepositoryUserLinkRecord mRepositoryUserLink;
    }
}
