﻿using System;
using System.Diagnostics;
using Bespoke.Common.Data;
using NUnit.Framework;

namespace Bespoke.ResearchExpert.Data.Test
{
    [TestFixture]
    public class UserGroupLinkManagerTest : TestFixtureBase
    {
        public static UserGroupLinkRecord CreateUserGroupLink(UserRecord user, GroupRecord group, int roleKey)
        {
            UserGroupLinkRecord ugl = new UserGroupLinkRecord(user.Key, group.Key, roleKey);
            UserGroupLinkManager.AddUserGroupLink(ugl);
            Trace.WriteLine("CreateUserGroupLink(): UserGroupLink added.");

            return ugl;
        }

        public static void DeleteUserGroupLink(UserGroupLinkRecord ugl)
        {
            UserGroupLinkManager.DeleteUserGroupLink(ugl);
            Trace.WriteLine("DeleteUserGroupLink(): UserGroupLink deleted.");

            UserGroupLinkRecord foundUserGroupLink;
            bool found = UserGroupLinkManager.RetrieveUserGroupLinkByKey(ugl.UserKey, ugl.GroupKey, out foundUserGroupLink);
            Assert.IsFalse(found, "DeleteUserGroupLink(): Delete failed.");
        }

        [SetUp]
        public void Setup()
        {
            mUser = UserManagerTest.CreateUser();
            mGroup = GroupManagerTest.CreateGroup(mUser);
            mUserGroupLink = CreateUserGroupLink(mUser, mGroup, mDatabase.Roles[Role.User].Key);
        }

        [TearDown]
        public void TearDown()
        {
            DeleteUserGroupLink(mUserGroupLink);
            GroupManagerTest.DeleteGroup(mGroup);
            UserManagerTest.DeleteUser(mUser);
        }

        [Test]
        public void AddUserGroupLink()
        {
            Trace.WriteLine("AddUserGroupLink(): Covered in TestFixture.Setup()");
        }

        [Test]
        public void DeleteUserGroupLink()
        {
            Trace.WriteLine("AddUserGroupLink(): Covered in TestFixture.TearDown()");
        }

        [Test]
        public void LoadUserGroupLinkByUserKey()
        {
            UserGroupLinkRecordList uglList = UserGroupLinkManager.LoadUserGroupLinkByUserKey(mUser.Key);
            Assert.IsTrue(uglList.Count > 0);

            Trace.WriteLine("LoadUserGroupLinkByUserKey(): " + uglList.Count + " User-Group Link Records loaded.");
        }

        [Test]
        public void LoadUserGroupLinkByGroupKey()
        {
            UserGroupLinkRecordList uglList = UserGroupLinkManager.LoadUserGroupLinkByGroupKey(mGroup.Key);
            Assert.IsTrue(uglList.Count > 0);

            Trace.WriteLine("LoadUserGroupLinkByGroupKey(): " + uglList.Count + " User-Group Link Records loaded.");
        }

        [Test]
        public void RetrieveUserGroupLinkByKey()
        {
            UserGroupLinkRecord foundUserGroupLink;
            bool found = UserGroupLinkManager.RetrieveUserGroupLinkByKey(mUser.Key, mGroup.Key, out foundUserGroupLink);
            Assert.IsTrue(found, "RetrieveUserGroupLinkByKey(): User-Group Link Record not found.");
            Assert.IsTrue(mUserGroupLink == foundUserGroupLink);

            Trace.WriteLine("RetrieveUserGroupLinkByKey(): User-Group Link Record retrieved.");
        }

        [Test]
        public void UpdateUserGroupLink()
        {
            mUserGroupLink.RoleKey = mDatabase.Roles[Role.Administrator].Key;
            UserGroupLinkManager.UpdateUserGroupLink(mUserGroupLink);

            UserGroupLinkRecord foundUserGroupLink;
            bool found = UserGroupLinkManager.RetrieveUserGroupLinkByKey(mUserGroupLink.UserKey, mUserGroupLink.GroupKey, out foundUserGroupLink);
            Assert.IsTrue(found, "UpdateUserGroupLink(): Post-update operation(s) failed.");
            Assert.AreEqual(mUserGroupLink.RoleKey, foundUserGroupLink.RoleKey);
        }


        private UserRecord mUser;
        private GroupRecord mGroup;
        private UserGroupLinkRecord mUserGroupLink;
    }
}
