﻿using System;
using System.Reflection;
using System.Collections.Generic;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public enum Role
    {
        Observer = 0,
        User = 1,
        Administrator = 2
    }

    /// <summary>
    /// 
    /// </summary>
    public enum DocumentType
    {
        AdobeAcrobat = 0
    }

    /// <summary>
    /// 
    /// </summary>
    public class Database : IDisposable
    {
        /// <summary>
        /// 
        /// </summary>
        public Dictionary<Role, RoleRecord> Roles
        {
            get
            {
                return mRoles;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public Dictionary<DocumentType, DocumentTypeRecord> DocumentTypes
        {
            get
            {
                return mDocumentTypes;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public AuthorRecord UnknownAuthor
        {
            get
            {
                return mUnknownAuthor;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public PublicationSourceRecord UnknownPublicationSource
        {
            get
            {
                return mUnknownPublicationSource;
            }
        }
        
        /// <summary>
        /// 
        /// </summary>
        public void Init(bool openPersistentConnection)
        {
            mDataProvider = new SqlDataProvider(ConnectionString);

            if (openPersistentConnection)
            {
                mDataProvider.OpenPersistentConnection();
            }

            // Initialize data managers
            Type[] types = Assembly.GetExecutingAssembly().GetTypes();
            foreach (Type type in types)
            {
                DataManagerAttribute[] dataManagerAttributes = (DataManagerAttribute[])type.GetCustomAttributes(typeof(DataManagerAttribute), true);
                if (dataManagerAttributes.Length > 0)
                {
                    MethodInfo initMethod = type.GetMethod(dataManagerAttributes[0].InitMethod);
                    if (initMethod != null)
                    {
                        object[] args = { mDataProvider };
                        initMethod.Invoke(null, args);
                    }
                }                
            }

            // Initialize user roles
            mRoles = new Dictionary<Role, RoleRecord>();
            RoleRecordList roleRecordList = RoleManager.LoadRole();            
            Role[] definedRoles = (Role[])Enum.GetValues(typeof(Role));
            foreach (Role definedRole in definedRoles)
            {
                int index = roleRecordList.IndexOf(definedRole.ToString());
                if (index != -1)
                {
                    mRoles.Add(definedRole, roleRecordList[index]);
                }
            }

            Assert.IsTrue("mRoles.Count", mRoles.Count == definedRoles.Length);

            // Initialize document types
            mDocumentTypes = new Dictionary<DocumentType, DocumentTypeRecord>(); 
            DocumentTypeRecordList documentTypeRecordList = DocumentTypeManager.LoadDocumentType();
            DocumentType[] definedDocumentTypes = (DocumentType[])Enum.GetValues(typeof(DocumentType));
            foreach (DocumentType definedDocumentType in definedDocumentTypes)
            {
                int index = documentTypeRecordList.IndexOf(definedDocumentType.ToString());
                if (index != -1)
                {
                    mDocumentTypes.Add(definedDocumentType, documentTypeRecordList[index]);
                }
            }

            Assert.IsTrue("mDocumentTypes.Count", mDocumentTypes.Count == definedDocumentTypes.Length);

            // Retrieve unknown author
            Assert.IsTrue("Unknown Author", AuthorManager.RetrieveAuthorByFullName("Unknown", out mUnknownAuthor));

            // Retrieve unknown publication source
            Assert.IsTrue("Unknown PublicationSource", PublicationSourceManager.RetrievePublicationSourceByName("Unknown", out mUnknownPublicationSource));
        }

        /// <summary>
        /// 
        /// </summary>
        public void Dispose()
        {
            Shutdown();
        }        

        /// <summary>
        /// 
        /// </summary>
        public void Shutdown()
        {
            mDataProvider.ClosePersistentConnection();
        }

        //TODO: read from config file
        private static readonly string ConnectionString = @"Data Source=localhost\SQLEXPRESS;Initial Catalog=ResearchExpert;Integrated Security=True";

        private SqlDataProvider mDataProvider;
        private Dictionary<Role, RoleRecord> mRoles;
        private Dictionary<DocumentType, DocumentTypeRecord> mDocumentTypes;
        private AuthorRecord mUnknownAuthor;
        private PublicationSourceRecord mUnknownPublicationSource;
    }
}
