﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class DocumentKeywordLinkManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dkl"></param>
        public static void AddDocumentKeywordLink(DocumentKeywordLinkRecord dkl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddDocumentKeywordLink";
                    sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, dkl.DocumentKey);
                    sDataProvider.AddParameter("@KeywordKey", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, dkl.KeywordKey);

                    sDataProvider.ExecuteNonQuery();
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="dkl"></param>
        public static void CreateDocumentKeywordLink(DocumentKeywordLinkRecord dkl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "CreateDocumentKeywordLink";
                    sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, dkl.DocumentKey);
                    sDataProvider.AddParameter("@KeywordKey", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, dkl.KeywordKey);

                    sDataProvider.ExecuteNonQuery();
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dkl"></param>
        /// <returns></returns>
        public static bool DeleteDocumentKeywordLink(DocumentKeywordLinkRecord dkl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteDocumentKeywordLink";
                    sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, dkl.DocumentKey);
                    sDataProvider.AddParameter("@KeywordKey", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, dkl.KeywordKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="keywordKey"></param>
        /// <returns></returns>
        public static DocumentKeywordLinkRecordList LoadDocumentKeywordLinkByDocumentKey(int documentKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentKeywordLinkByDocumentKey";
                sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, documentKey);

                return RetrieveMultipleDocumentKeywordLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="keywordKey"></param>
        /// <returns></returns>
        public static DocumentKeywordLinkRecordList LoadDocumentKeywordLinkByKeywordKey(int keywordKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentKeywordLinkByKeywordKey";
                sDataProvider.AddParameter("@KeywordKey", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, keywordKey);

                return RetrieveMultipleDocumentKeywordLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="documentKey"></param>
        /// <param name="keywordKey"></param>
        /// <param name="dkl"></param>
        /// <returns></returns>
        public static bool RetrieveDocumentKeywordLinkByKey(int documentKey, int keywordKey, out DocumentKeywordLinkRecord dkl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveDocumentKeywordLinkByKey";
                sDataProvider.AddParameter("@DocumentKey", SqlDbType.Int, ParameterDirection.Input, DocumentRecord.KeySize, documentKey);
                sDataProvider.AddParameter("@KeywordKey", SqlDbType.Int, ParameterDirection.Input, KeywordRecord.KeySize, keywordKey);

                return RetrieveDocumentKeywordLink(out dkl);
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the keyword group link.
        /// </summary>
        /// <param name="dkl">The dkl.</param>
        /// <returns></returns>
        private static bool RetrieveDocumentKeywordLink(out DocumentKeywordLinkRecord dkl)
        {
            try
            {
                dkl = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        dkl = new DocumentKeywordLinkRecord();
                        dkl.DocumentKey = sDataProvider.ReadValue<int>("DocumentKey");
                        dkl.KeywordKey = sDataProvider.ReadValue<int>("KeywordKey");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the keywords.
        /// </summary>
        /// <returns></returns>
        private static DocumentKeywordLinkRecordList RetrieveMultipleDocumentKeywordLinkRecords()
        {
            DocumentKeywordLinkRecordList dklList = new DocumentKeywordLinkRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        DocumentKeywordLinkRecord dkl = new DocumentKeywordLinkRecord();
                        dkl.DocumentKey = sDataProvider.ReadValue<int>("DocumentKey");
                        dkl.KeywordKey = sDataProvider.ReadValue<int>("KeywordKey");

                        dklList.Add(dkl);
                    }

                    return dklList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}