﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class DocumentTypeManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Retrieve all documentTypes in the database.
        /// </summary>
        /// <returns>The retrieved documentType collection.</returns>
        public static DocumentTypeRecordList LoadDocumentType()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadDocumentType";

                return RetrieveMultipleDocumentTypes();
            }
        }

        /// <summary>
        /// Retrieves the documentType by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="documentType">The documentType.</param>
        /// <returns></returns>
        public static bool RetrieveDocumentTypeByKey(int key, out DocumentTypeRecord documentType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveDocumentTypeByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, DocumentTypeRecord.KeySize, key);

                return RetrieveDocumentType(out documentType);
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the documentType.
        /// </summary>
        /// <param name="documentType">The documentType.</param>
        /// <returns></returns>
        private static bool RetrieveDocumentType(out DocumentTypeRecord documentType)
        {
            try
            {
                documentType = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        documentType = new DocumentTypeRecord();
                        documentType.Key = sDataProvider.ReadValue<int>("Key");
                        documentType.Name = sDataProvider.ReadValue<string>("Name");
                        documentType.FileExtension = sDataProvider.ReadValue<string>("FileExtension");
                        documentType.HttpContentType = sDataProvider.ReadValue<string>("HttpContentType");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the documentTypes.
        /// </summary>
        /// <returns></returns>
        private static DocumentTypeRecordList RetrieveMultipleDocumentTypes()
        {
            DocumentTypeRecordList documentTypeList = new DocumentTypeRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        DocumentTypeRecord documentType = new DocumentTypeRecord();
                        documentType.Key = sDataProvider.ReadValue<int>("Key");
                        documentType.Name = sDataProvider.ReadValue<string>("Name");
                        documentType.FileExtension = sDataProvider.ReadValue<string>("FileExtension");
                        documentType.HttpContentType = sDataProvider.ReadValue<string>("HttpContentType");

                        documentTypeList.Add(documentType);
                    }

                    return documentTypeList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}