﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class PublicationSourceTypeManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Adds the publicationSourceType.
        /// </summary>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        public static void AddPublicationSourceType(PublicationSourceTypeRecord publicationSourceType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddPublicationSourceType";
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceTypeRecord.NameSize, publicationSourceType.Name);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    publicationSourceType.Key = (int)returnValue.Value;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// Deletes the publicationSourceType.
        /// </summary>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        /// <returns></returns>
        public static bool DeletePublicationSourceType(PublicationSourceTypeRecord publicationSourceType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeletePublicationSourceType";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, publicationSourceType.Key);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// Retrieve all publicationSourceTypes in the database.
        /// </summary>
        /// <returns>The retrieved publicationSourceType collection.</returns>
        public static PublicationSourceTypeRecordList LoadPublicationSourceType()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadPublicationSourceType";

                return RetrieveMultiplePublicationSourceTypes();
            }
        }

        /// <summary>
        /// Retrieves the publicationSourceType by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        /// <returns></returns>
        public static bool RetrievePublicationSourceTypeByKey(int key, out PublicationSourceTypeRecord publicationSourceType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrievePublicationSourceTypeByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, PublicationSourceTypeRecord.KeySize, key);

                return RetrievePublicationSourceType(out publicationSourceType);
            }
        }

        /// <summary>
        /// Retrieves the publicationSourceType by name.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        /// <returns></returns>
        public static bool RetrievePublicationSourceTypeByName(string name, out PublicationSourceTypeRecord publicationSourceType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrievePublicationSourceTypeByName";
                sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceTypeRecord.NameSize, name);

                return RetrievePublicationSourceType(out publicationSourceType);
            }
        }

        /// <summary>
        /// Updates the publicationSourceType.
        /// </summary>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        /// <returns></returns>
        public static bool UpdatePublicationSourceType(PublicationSourceTypeRecord publicationSourceType)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdatePublicationSourceType";
                    sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, PublicationSourceTypeRecord.KeySize, publicationSourceType.Key);
                    sDataProvider.AddParameter("@Name", SqlDbType.VarChar, ParameterDirection.Input, PublicationSourceTypeRecord.NameSize, publicationSourceType.Name);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the publicationSourceType.
        /// </summary>
        /// <param name="publicationSourceType">The publicationSourceType.</param>
        /// <returns></returns>
        private static bool RetrievePublicationSourceType(out PublicationSourceTypeRecord publicationSourceType)
        {
            try
            {
                publicationSourceType = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        publicationSourceType = new PublicationSourceTypeRecord();
                        publicationSourceType.Key = sDataProvider.ReadValue<int>("Key");
                        publicationSourceType.Name = sDataProvider.ReadValue<string>("Name");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the publicationSourceTypes.
        /// </summary>
        /// <returns></returns>
        private static PublicationSourceTypeRecordList RetrieveMultiplePublicationSourceTypes()
        {
            PublicationSourceTypeRecordList publicationSourceTypeList = new PublicationSourceTypeRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        PublicationSourceTypeRecord publicationSourceType = new PublicationSourceTypeRecord();
                        publicationSourceType.Key = sDataProvider.ReadValue<int>("Key");
                        publicationSourceType.Name = sDataProvider.ReadValue<string>("Name");

                        publicationSourceTypeList.Add(publicationSourceType);
                    }

                    return publicationSourceTypeList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}