﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class RepositoryGroupLinkManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="group"></param>
        public static void AddRepositoryGroupLink(RepositoryGroupLinkRecord rgl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddRepositoryGroupLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, rgl.RepositoryKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, rgl.GroupKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, rgl.RoleKey);

                    sDataProvider.ExecuteNonQuery();
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rgl"></param>
        /// <returns></returns>
        public static bool DeleteRepositoryGroupLink(RepositoryGroupLinkRecord rgl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteRepositoryGroupLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, rgl.RepositoryKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, rgl.GroupKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="groupKey"></param>
        /// <returns></returns>
        public static RepositoryGroupLinkRecordList LoadRepositoryGroupLinkByRepositoryKey(int repositoryKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepositoryGroupLinkByRepositoryKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, repositoryKey);

                return RetrieveMultipleRepositoryGroupLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="groupKey"></param>
        /// <returns></returns>
        public static RepositoryGroupLinkRecordList LoadRepositoryGroupLinkByGroupKey(int groupKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepositoryGroupLinkByGroupKey";
                sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, groupKey);

                return RetrieveMultipleRepositoryGroupLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="repositoryKey"></param>
        /// <param name="groupKey"></param>
        /// <param name="rgl"></param>
        /// <returns></returns>
        public static bool RetrieveRepositoryGroupLinkByKey(int repositoryKey, int groupKey, out RepositoryGroupLinkRecord rgl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveRepositoryGroupLinkByKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, repositoryKey);
                sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, groupKey);

                return RetrieveRepositoryGroupLink(out rgl);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rgl"></param>
        /// <returns></returns>
        public static bool UpdateRepositoryGroupLink(RepositoryGroupLinkRecord rgl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateRepositoryGroupLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, rgl.RepositoryKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, rgl.GroupKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, rgl.RoleKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the group group link.
        /// </summary>
        /// <param name="rgl">The rgl.</param>
        /// <returns></returns>
        private static bool RetrieveRepositoryGroupLink(out RepositoryGroupLinkRecord rgl)
        {
            try
            {
                rgl = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        rgl = new RepositoryGroupLinkRecord();
                        rgl.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        rgl.GroupKey = sDataProvider.ReadValue<int>("GroupKey");
                        rgl.RoleKey = sDataProvider.ReadValue<int>("RoleKey");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the groups.
        /// </summary>
        /// <returns></returns>
        private static RepositoryGroupLinkRecordList RetrieveMultipleRepositoryGroupLinkRecords()
        {
            RepositoryGroupLinkRecordList rglList = new RepositoryGroupLinkRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        RepositoryGroupLinkRecord rgl = new RepositoryGroupLinkRecord();
                        rgl.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        rgl.GroupKey = sDataProvider.ReadValue<int>("GroupKey");
                        rgl.RoleKey = sDataProvider.ReadValue<int>("RoleKey");

                        rglList.Add(rgl);
                    }

                    return rglList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}