﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class RepositoryUserLinkManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="user"></param>
        public static void AddRepositoryUserLink(RepositoryUserLinkRecord rul)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddRepositoryUserLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, rul.RepositoryKey);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, rul.UserKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, rul.RoleKey);

                    sDataProvider.ExecuteNonQuery();
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rul"></param>
        /// <returns></returns>
        public static bool DeleteRepositoryUserLink(RepositoryUserLinkRecord rul)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteRepositoryUserLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, rul.RepositoryKey);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, rul.UserKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userKey"></param>
        /// <returns></returns>
        public static RepositoryUserLinkRecordList LoadRepositoryUserLinkByRepositoryKey(int repositoryKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepositoryUserLinkByRepositoryKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);

                return RetrieveMultipleRepositoryUserLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userKey"></param>
        /// <returns></returns>
        public static RepositoryUserLinkRecordList LoadRepositoryUserLinkByUserKey(int userKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRepositoryUserLinkByUserKey";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);

                return RetrieveMultipleRepositoryUserLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="repositoryKey"></param>
        /// <param name="userKey"></param>
        /// <param name="rul"></param>
        /// <returns></returns>
        public static bool RetrieveRepositoryUserLinkByKey(int repositoryKey, int userKey, out RepositoryUserLinkRecord rul)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveRepositoryUserLinkByKey";
                sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, repositoryKey);
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);

                return RetrieveRepositoryUserLink(out rul);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rul"></param>
        /// <returns></returns>
        public static bool UpdateRepositoryUserLink(RepositoryUserLinkRecord rul)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateRepositoryUserLink";
                    sDataProvider.AddParameter("@RepositoryKey", SqlDbType.Int, ParameterDirection.Input, RepositoryRecord.KeySize, rul.RepositoryKey);
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, rul.UserKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, rul.RoleKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the user group link.
        /// </summary>
        /// <param name="rul">The rul.</param>
        /// <returns></returns>
        private static bool RetrieveRepositoryUserLink(out RepositoryUserLinkRecord rul)
        {
            try
            {
                rul = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        rul = new RepositoryUserLinkRecord();
                        rul.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        rul.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        rul.RoleKey = sDataProvider.ReadValue<int>("RoleKey");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the users.
        /// </summary>
        /// <returns></returns>
        private static RepositoryUserLinkRecordList RetrieveMultipleRepositoryUserLinkRecords()
        {
            RepositoryUserLinkRecordList rulList = new RepositoryUserLinkRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        RepositoryUserLinkRecord rul = new RepositoryUserLinkRecord();
                        rul.RepositoryKey = sDataProvider.ReadValue<int>("RepositoryKey");
                        rul.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        rul.RoleKey = sDataProvider.ReadValue<int>("RoleKey");

                        rulList.Add(rul);
                    }

                    return rulList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}