﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class RoleManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// Retrieve all roles in the database.
        /// </summary>
        /// <returns>The retrieved role collection.</returns>
        public static RoleRecordList LoadRole()
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadRole";

                return RetrieveMultipleRoles();
            }
        }

        /// <summary>
        /// Retrieves the role by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="role">The role.</param>
        /// <returns></returns>
        public static bool RetrieveRoleByKey(int key, out RoleRecord role)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveRoleByKey";
                sDataProvider.AddParameter("@Key", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, key);

                return RetrieveRole(out role);
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the role.
        /// </summary>
        /// <param name="role">The role.</param>
        /// <returns></returns>
        private static bool RetrieveRole(out RoleRecord role)
        {
            try
            {
                role = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        role = new RoleRecord();
                        role.Key = sDataProvider.ReadValue<int>("Key");
                        role.Name = sDataProvider.ReadValue<string>("Name");
                        role.Description = sDataProvider.ReadValue<string>("Description");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the roles.
        /// </summary>
        /// <returns></returns>
        private static RoleRecordList RetrieveMultipleRoles()
        {
            RoleRecordList roleList = new RoleRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        RoleRecord role = new RoleRecord();
                        role.Key = sDataProvider.ReadValue<int>("Key");
                        role.Name = sDataProvider.ReadValue<string>("Name");
                        role.Description = sDataProvider.ReadValue<string>("Description");

                        roleList.Add(role);
                    }

                    return roleList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}