﻿using System;
using System.Data;
using System.Data.SqlClient;
using Bespoke.Common;
using Bespoke.Common.Data;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    [DataManager("Init")]
    public static class UserGroupLinkManager
    {
        /// <summary>
        /// Inits the specified data provider.
        /// </summary>
        /// <param name="dataProvider">The data provider.</param>
        public static void Init(SqlDataProvider dataProvider)
        {
            sDataProvider = dataProvider;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="user"></param>
        public static void AddUserGroupLink(UserGroupLinkRecord ugl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "AddUserGroupLink";
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, ugl.UserKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, ugl.GroupKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, ugl.RoleKey);

                    sDataProvider.ExecuteNonQuery();
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }  
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ugl"></param>
        /// <returns></returns>
        public static bool DeleteUserGroupLink(UserGroupLinkRecord ugl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "DeleteUserGroupLink";
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, ugl.UserKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, ugl.GroupKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }                
            }
            finally
            {
                sDataProvider.ResetQuery();
            }            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userKey"></param>
        /// <returns></returns>
        public static UserGroupLinkRecordList LoadUserGroupLinkByUserKey(int userKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadUserGroupLinkByUserKey";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);

                return RetrieveMultipleUserGroupLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="groupKey"></param>
        /// <returns></returns>
        public static UserGroupLinkRecordList LoadUserGroupLinkByGroupKey(int groupKey)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "LoadUserGroupLinkByGroupKey";
                sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, groupKey);

                return RetrieveMultipleUserGroupLinkRecords();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userKey"></param>
        /// <param name="groupKey"></param>
        /// <param name="ugl"></param>
        /// <returns></returns>
        public static bool RetrieveUserGroupLinkByKey(int userKey, int groupKey, out UserGroupLinkRecord ugl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            lock (sDataProvider)
            {
                sDataProvider.CommandText = "RetrieveUserGroupLinkByKey";
                sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, userKey);
                sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, groupKey);

                return RetrieveUserGroupLink(out ugl);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ugl"></param>
        /// <returns></returns>
        public static bool UpdateUserGroupLink(UserGroupLinkRecord ugl)
        {
            Assert.ParamIsNotNull(sDataProvider);

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.CommandText = "UpdateUserGroupLink";
                    sDataProvider.AddParameter("@UserKey", SqlDbType.Int, ParameterDirection.Input, UserRecord.KeySize, ugl.UserKey);
                    sDataProvider.AddParameter("@GroupKey", SqlDbType.Int, ParameterDirection.Input, GroupRecord.KeySize, ugl.GroupKey);
                    sDataProvider.AddParameter("@RoleKey", SqlDbType.Int, ParameterDirection.Input, RoleRecord.KeySize, ugl.RoleKey);
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();

                    sDataProvider.ExecuteNonQuery();

                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #region Private Methods

        /// <summary>
        /// Retrieves the user group link.
        /// </summary>
        /// <param name="ugl">The ugl.</param>
        /// <returns></returns>
        private static bool RetrieveUserGroupLink(out UserGroupLinkRecord ugl)
        {
            try
            {
                ugl = null;

                lock (sDataProvider)
                {
                    SqlParameter returnValue = sDataProvider.AddReturnValueParameter();
                    sDataProvider.ExecuteQuery();

                    if (sDataProvider.DataReader.Read())
                    {
                        ugl = new UserGroupLinkRecord();
                        ugl.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        ugl.GroupKey = sDataProvider.ReadValue<int>("GroupKey");
                        ugl.RoleKey = sDataProvider.ReadValue<int>("RoleKey");
                    }

                    sDataProvider.EndQuery();
                    return Convert.ToBoolean(returnValue.Value);
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        /// <summary>
        /// Loads the users.
        /// </summary>
        /// <returns></returns>
        private static UserGroupLinkRecordList RetrieveMultipleUserGroupLinkRecords()
        {
            UserGroupLinkRecordList uglList = new UserGroupLinkRecordList();

            try
            {
                lock (sDataProvider)
                {
                    sDataProvider.ExecuteQuery();

                    while (sDataProvider.DataReader.Read())
                    {
                        UserGroupLinkRecord ugl = new UserGroupLinkRecord();
                        ugl.UserKey = sDataProvider.ReadValue<int>("UserKey");
                        ugl.GroupKey = sDataProvider.ReadValue<int>("GroupKey");
                        ugl.RoleKey = sDataProvider.ReadValue<int>("RoleKey");

                        uglList.Add(ugl);
                    }

                    return uglList;
                }
            }
            finally
            {
                sDataProvider.ResetQuery();
            }
        }

        #endregion

        private static SqlDataProvider sDataProvider;
    }
}