﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class AuthorRecord
    {
        public static readonly int KeySize = 4;
        public static readonly int FullNameSize = 60;
        public static readonly int AffiliationSize = 60;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the full name.
        /// </summary>
        /// <value>The full name.</value>
        public string FullName
        {
            get
            {
                return mFullName;
            }
            set
            {
                mFullName = value;
            }
        }

        /// <summary>
        /// Gets or sets the affiliation.
        /// </summary>
        /// <value>The affiliation.</value>
        public string Affiliation
        {
            get
            {
                return mAffiliation;
            }
            set
            {
                mAffiliation = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AuthorRecord"/> class.
        /// </summary>
        public AuthorRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AuthorRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="fullName">The full name.</param>
        /// <param name="affiliation">The affiliation.</param>
        public AuthorRecord(int key, string fullName, string affiliation)
        {
            mKey = key;
            mFullName = fullName;
            mAffiliation = affiliation;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(AuthorRecord lhs, AuthorRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(AuthorRecord lhs, AuthorRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is AuthorRecord))
            {
                return false;
            }

            return this == (AuthorRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mFullName;
        }

        private int mKey;
        private string mFullName;
        private string mAffiliation;
    }

    /// <summary>
    /// 
    /// </summary>
    public class AuthorRecordList : List<AuthorRecord> { }
}
