﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class DocumentRecord
    {
        public static readonly int KeySize = 4;    
        public static readonly int TitleSize = 255;
        public static readonly int PublicationDateSize = 4;
        public static readonly int DescriptionSize = 255;
        public static readonly int AbstractSize = int.MaxValue;
        public static readonly int FileNameSize = 255;
        public static readonly int DataSize = int.MaxValue;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the document type key.
        /// </summary>
        /// <value>The document type key.</value>
        public int DocumentTypeKey
        {
            get
            {
                return mDocumentTypeKey;
            }
            set
            {
                mDocumentTypeKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the repository key.
        /// </summary>
        /// <value>The repository key.</value>
        public int RepositoryKey
        {
            get
            {
                return mRepositoryKey;
            }
            set
            {
                mRepositoryKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return mTitle;
            }
            set
            {
                mTitle = value;
            }
        }

        /// <summary>
        /// Gets or sets the author key.
        /// </summary>
        /// <value>The author key.</value>
        public int AuthorKey
        {
            get
            {
                return mAuthorKey;
            }
            set
            {
                mAuthorKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the publication source key.
        /// </summary>
        /// <value>The publication source key.</value>
        public int PublicationSourceKey
        {
            get
            {
                return mPublicationSourceKey;
            }
            set
            {
                mPublicationSourceKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the creation date.
        /// </summary>
        /// <value>The creation date.</value>
        public DateTime PublicationDate
        {
            get
            {
                return mPublicationDate;
            }
            set
            {
                mPublicationDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the abstract.
        /// </summary>
        /// <value>The abstract.</value>
        public string Abstract
        {
            get
            {
                return mAbstract;
            }
            set
            {
                mAbstract = value;
            }
        }

        /// <summary>
        /// Gets or sets the name of the file.
        /// </summary>
        /// <value>The name of the file.</value>
        public string FileName
        {
            get
            {
                return mFileName;
            }
            set
            {
                mFileName = value;
            }
        }

        /// <summary>
        /// Gets or sets the data.
        /// </summary>
        /// <value>The data.</value>
        public byte[] Data
        {
            get
            {
                return mData;
            }
            set
            {
                mData = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentRecord"/> class.
        /// </summary>
        public DocumentRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="documentTypeKey">The document type key.</param>
        /// <param name="repositoryKey">The repository key.</param>
        /// <param name="title">The title.</param>
        /// <param name="authorKey">The author key.</param>
        /// <param name="publicationSourceKey">The publication source key.</param>
        /// <param name="publicationDate">The publication date.</param>        
        /// <param name="abstractData">The abstract data.</param>
        /// <param name="fileName">Name of the file.</param>
        /// <param name="data">The data.</param>
        public DocumentRecord(int key, int documentTypeKey, int repositoryKey, string title, int authorKey, int publicationSourceKey, DateTime publicationDate, string abstractData, string fileName, byte[] data)
        {
            mKey = key;
            mDocumentTypeKey = documentTypeKey;
            mRepositoryKey = repositoryKey;
            mTitle = title;
            mAuthorKey = authorKey;
            mPublicationSourceKey = publicationSourceKey;
            mPublicationDate = publicationDate;
            mAbstract = abstractData;
            mFileName = fileName;
            mData = data;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(DocumentRecord lhs, DocumentRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(DocumentRecord lhs, DocumentRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is DocumentRecord))
            {
                return false;
            }

            return this == (DocumentRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mTitle;
        }

        private int mKey;
        private int mDocumentTypeKey;
        private int mRepositoryKey;
        private string mTitle;
        private int mAuthorKey;
        private int mPublicationSourceKey;
        private DateTime mPublicationDate;
        private string mAbstract;
        private string mFileName;
        private byte[] mData;
    }

    /// <summary>
    /// 
    /// </summary>
    public class DocumentRecordList : List<DocumentRecord> { }
}
