﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class DocumentTypeRecord
    {
        public static readonly int KeySize = 4;
        public static readonly int NameSize = 30;
        public static readonly int FileExtensionSize = 10;
        public static readonly int HttpContentTypeSize = 30;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                mName = value;
            }
        }

        /// <summary>
        /// Gets or sets the fileExtension.
        /// </summary>
        /// <value>The fileExtension.</value>
        public string FileExtension
        {
            get
            {
                return mFileExtension;
            }
            set
            {
                mFileExtension = value;
            }
        }

        /// <summary>
        /// Gets or sets the type of the HTTP content.
        /// </summary>
        /// <value>The type of the HTTP content.</value>
        public string HttpContentType
        {
            get
            {
                return mHttpContentType;
            }
            set
            {
                mHttpContentType = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentTypeRecord"/> class.
        /// </summary>
        public DocumentTypeRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentTypeRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="name">The name.</param>
        /// <param name="fileExtension">The fileExtension.</param>
        /// <param name="httpContentType">Type of the HTTP content.</param>
        public DocumentTypeRecord(int key, string name, string fileExtension, string httpContentType)
        {
            mKey = key;
            mName = name;
            mFileExtension = fileExtension;
            mHttpContentType = httpContentType;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(DocumentTypeRecord lhs, DocumentTypeRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(DocumentTypeRecord lhs, DocumentTypeRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is DocumentTypeRecord))
            {
                return false;
            }

            return this == (DocumentTypeRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mName;
        }

        private int mKey;
        private string mName;
        private string mFileExtension;
        private string mHttpContentType;
    }

    /// <summary>
    /// 
    /// </summary>
    public class DocumentTypeRecordList : List<DocumentTypeRecord>
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="documentTypeName"></param>
        /// <returns></returns>
        public int IndexOf(string documentTypeName)
        {
            for (int index = 0; index < Count; index++)
            {
                if (this[index].Name == documentTypeName)
                {
                    return index;
                }
            }

            return -1;
        }
    }
}
