﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class RepositoryRecord
    {
        public static readonly int KeySize = 4;    
        public static readonly int NameSize = 30;
        public static readonly int CreationDateSize = 8;
        public static readonly int DescriptionSize = 255;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the user key.
        /// </summary>
        /// <value>The user key.</value>
        public int UserKey
        {
            get
            {
                return mUserKey;
            }
            set
            {
                mUserKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the creation date.
        /// </summary>
        /// <value>The creation date.</value>
        public DateTime CreationDate
        {
            get
            {
                return mCreationDate;
            }
            set
            {
                mCreationDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                mName = value;
            }
        }

        /// <summary>
        /// Gets or sets the full name.
        /// </summary>
        /// <value>The full name.</value>
        public string Description
        {
            get
            {
                return mDescription;
            }
            set
            {
                mDescription = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RepositoryRecord"/> class.
        /// </summary>
        public RepositoryRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RepositoryRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="userKey">The key of the associated user.</param>
        /// <param name="creationDate">The creation date.</param>
        /// <param name="name">The name.</param>
        /// <param name="description">The description.</param>
        public RepositoryRecord(int key, int userKey, DateTime creationDate, string name, string description)
        {
            mKey = key;
            mUserKey = userKey;
            mCreationDate = creationDate;
            mName = name;
            mDescription = description;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(RepositoryRecord lhs, RepositoryRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(RepositoryRecord lhs, RepositoryRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is RepositoryRecord))
            {
                return false;
            }

            return this == (RepositoryRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mName;
        }

        private int mKey;
        private int mUserKey;
        private DateTime mCreationDate;
        private string mName;
        private string mDescription;
    }

    /// <summary>
    /// 
    /// </summary>
    public class RepositoryRecordList : List<RepositoryRecord> { }
}
