﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class ReviewRecord
    {
        public static readonly int KeySize = 4;    
        public static readonly int CreationDateSize = 8;
        public static readonly int RatingSize = 1;
        public static readonly int CommentsSize = int.MaxValue;
        public static readonly int DataSize = int.MaxValue;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the document key.
        /// </summary>
        /// <value>The document key.</value>
        public int DocumentKey
        {
            get
            {
                return mDocumentKey;
            }
            set
            {
                mDocumentKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the creation date.
        /// </summary>
        /// <value>The creation date.</value>
        public DateTime CreationDate
        {
            get
            {
                return mCreationDate;
            }
            set
            {
                mCreationDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the user key.
        /// </summary>
        /// <value>The user key.</value>
        public int UserKey
        {
            get
            {
                return mUserKey;
            }
            set
            {
                mUserKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the rating.
        /// </summary>
        /// <value>The rating.</value>
        public int Rating
        {
            get
            {
                return mRating;
            }
            set
            {
                mRating = value;
            }
        }

        /// <summary>
        /// Gets or sets the comments.
        /// </summary>
        /// <value>The comments.</value>
        public string Comments
        {
            get
            {
                return mComments;
            }
            set
            {
                mComments = value;
            }
        }

        /// <summary>
        /// Gets or sets the data.
        /// </summary>
        /// <value>The data.</value>
        public byte[] Data
        {
            get
            {
                return mData;
            }
            set
            {
                mData = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ReviewRecord"/> class.
        /// </summary>
        public ReviewRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ReviewRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="creationDate">The creation date.</param>
        /// <param name="userKey">The user key.</param>
        /// <param name="rating">The rating.</param>
        /// <param name="comments">The comments.</param>
        /// <param name="data">The data.</param>
        public ReviewRecord(int key, int documentKey, DateTime creationDate, int userKey, int rating, string comments, byte[] data)
        {
            mKey = key;
            mDocumentKey = documentKey;
            mCreationDate = creationDate;
            mUserKey = userKey;
            mRating = rating;
            mComments = comments;
            mData = data;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(ReviewRecord lhs, ReviewRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(ReviewRecord lhs, ReviewRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is ReviewRecord))
            {
                return false;
            }

            return this == (ReviewRecord)other;
        }

        private int mKey;
        private int mDocumentKey;
        private DateTime mCreationDate;
        private int mUserKey;        
        private int mRating;
        private string mComments;
        private byte[] mData;
    }

    /// <summary>
    /// 
    /// </summary>
    public class ReviewRecordList : List<ReviewRecord> { }
}
