﻿using System;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class RoleRecord
    {
        public static readonly int KeySize = 4;
        public static readonly int NameSize = 30;
        public static readonly int DescriptionSize = 255;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                mName = value;
            }
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get
            {
                return mDescription;
            }
            set
            {
                mDescription = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RoleRecord"/> class.
        /// </summary>
        public RoleRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RoleRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="name">The name.</param>
        /// <param name="description">The description.</param>
        public RoleRecord(int key, string name, string description)
        {
            mKey = key;
            mName = name;
            mDescription = description;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(RoleRecord lhs, RoleRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(RoleRecord lhs, RoleRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is RoleRecord))
            {
                return false;
            }

            return this == (RoleRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mName;
        }

        private int mKey;
        private string mName;
        private string mDescription;
    }

    /// <summary>
    /// 
    /// </summary>
    public class RoleRecordList : List<RoleRecord>
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="roleName"></param>
        /// <returns></returns>
        public int IndexOf(string roleName)
        {
            for (int index = 0; index < Count; index++)
            {
                if (this[index].Name == roleName)
                {
                    return index;
                }
            }

            return -1;
        }
    }
}
