﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using Bespoke.Common;

namespace Bespoke.ResearchExpert.Data
{
    /// <summary>
    /// 
    /// </summary>
    public class UserRecord
    {
        public static readonly int KeySize = 4;
        public static readonly int UsernameSize = 30;
        public static readonly int PasswordSize = 30;
        public static readonly int IsSiteAdministratorSize = 1;
        public static readonly int FullNameSize = 60;
        public static readonly int EmailAddressSize = 128;
        public static readonly int UrlSize = 255;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        public int Key
        {
            get
            {
                return mKey;
            }
            set
            {
                mKey = value;
            }
        }

        /// <summary>
        /// Gets or sets the username.
        /// </summary>
        /// <value>The username.</value>
        public string Username
        {
            get
            {
                return mUsername;
            }
            set
            {
                mUsername = value;
            }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        public string Password
        {
            get
            {
                return mPassword;
            }
            set
            {
                mPassword = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is site administrator.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is site administrator; otherwise, <c>false</c>.
        /// </value>
        public bool IsSiteAdministrator
        {
            get
            {
                return mIsSiteAdministrator;
            }
            set
            {
                mIsSiteAdministrator = value;
            }
        }

        /// <summary>
        /// Gets or sets the full name.
        /// </summary>
        /// <value>The full name.</value>
        public string FullName
        {
            get
            {
                return mFullName;
            }
            set
            {
                mFullName = value;
            }
        }

        /// <summary>
        /// Gets or sets the email address.
        /// </summary>
        /// <value>The email address.</value>
        public string EmailAddress
        {
            get
            {
                return mEmailAddress;
            }
            set
            {
                mEmailAddress = value;
            }
        }

        /// <summary>
        /// Gets or sets the URL.
        /// </summary>
        /// <value>The URL.</value>
        public string Url
        {
            get
            {
                return mUrl;
            }
            set
            {
                mUrl = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserRecord"/> class.
        /// </summary>
        public UserRecord()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserRecord"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="username">The username.</param>
        /// <param name="isSiteAdministrator"></param>
        /// <param name="password">The password.</param>
        /// <param name="fullName">The full name.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="url">The url.</param>
        public UserRecord(int key, string username, string password, bool isSiteAdministrator, string fullName, string emailAddress, string url)
        {
            mKey = key;
            mUsername = username;
            mPassword = password;
            mIsSiteAdministrator = isSiteAdministrator;
            mFullName = fullName;
            mEmailAddress = emailAddress;
            mUrl = url;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(UserRecord lhs, UserRecord rhs)
        {
            return RecordUtility.Equals(lhs, rhs);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="lhs">The LHS.</param>
        /// <param name="rhs">The RHS.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(UserRecord lhs, UserRecord rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public override bool Equals(object other)
        {
            if (!(other is UserRecord))
            {
                return false;
            }

            return this == (UserRecord)other;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return mUsername;
        }

        private int mKey;
        private string mUsername;
        private string mPassword;
        private bool mIsSiteAdministrator;
        private string mFullName;
        private string mEmailAddress;
        private string mUrl;
    }

    /// <summary>
    /// 
    /// </summary>
    public class UserRecordList : List<UserRecord> { }
}
