CREATE PROCEDURE [dbo].AddDocument (
	@DocumentTypeKey int,
	@RepositoryKey int,
	@Title varchar(255),
	@AuthorKey int,
	@PublicationSourceKey int,
	@PublicationDate datetime,
	@Abstract varchar(max),
	@FileName varchar(255),
	@Data varbinary(max)
)
AS

INSERT INTO Document (
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
)
VALUES (
	@DocumentTypeKey,
	@RepositoryKey,
	@Title,
	@AuthorKey,
	@PublicationSourceKey,
	@PublicationDate,
	@Abstract,
	@FileName,
	@Data
)

RETURN @@IDENTITY
GO

CREATE PROCEDURE [dbo].DeleteDocument (
	@Key int
)
AS

DELETE FROM Document
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].GetDocumentCount (
	@RepositoryKey int
)
AS

DECLARE @DocumentCount int

SELECT
	@DocumentCount = Count(*)
FROM Document
WHERE RepositoryKey = @RepositoryKey

RETURN @DocumentCount
GO

CREATE PROCEDURE [dbo].RetrieveDocumentByKey (
	@Key int
)
AS

SELECT
	[Key],
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
FROM Document
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].LoadDocument(
	@RepositoryKey int
)
AS

SELECT
	[Key],
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
FROM Document
WHERE RepositoryKey = @RepositoryKey
ORDER BY Title
GO

CREATE PROCEDURE [dbo].LoadDocumentByDocumentTypeKey (
	@RepositoryKey int,
	@DocumentTypeKey int
)
AS

SELECT
	[Key],
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
FROM Document
WHERE RepositoryKey = @RepositoryKey AND DocumentTypeKey = @DocumentTypeKey
ORDER BY Title
GO

CREATE PROCEDURE [dbo].LoadDocumentByAuthorKey (
	@RepositoryKey int,
	@AuthorKey int
)
AS

SELECT
	[Key],
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
FROM Document
WHERE RepositoryKey = @RepositoryKey AND AuthorKey = @AuthorKey
ORDER BY Title
GO

CREATE PROCEDURE [dbo].LoadDocumentByPublicationSourceKey (
	@RepositoryKey int,
	@PublicationSourceKey int
)
AS

SELECT
	[Key],
	DocumentTypeKey,
	RepositoryKey,
	Title,
	AuthorKey,
	PublicationSourceKey,
	PublicationDate,
	Abstract,
	FileName,
	Data
FROM Document
WHERE RepositoryKey = @RepositoryKey AND PublicationSourceKey = @PublicationSourceKey
ORDER BY Title
GO

CREATE PROCEDURE [dbo].UpdateDocument (
	@Key int,
	@DocumentTypeKey int,
	@RepositoryKey int,
	@Title varchar(255),
	@AuthorKey int,
	@PublicationSourceKey int,
	@PublicationDate datetime,
	@Abstract varchar(max),
	@FileName varchar(255),
	@Data varbinary(max)
)
AS

UPDATE Document
SET	DocumentTypeKey = @DocumentTypeKey,
	RepositoryKey = @RepositoryKey,
	Title = @Title,
	AuthorKey = @AuthorKey,
	PublicationSourceKey = @PublicationSourceKey,
	PublicationDate = @PublicationDate,
	Abstract = @Abstract,
	FileName = @FileName,
	Data = @Data
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO
