CREATE PROCEDURE [dbo].AddDocumentKeywordLink (
	@DocumentKey int,
	@KeywordKey int
)
AS

INSERT INTO DocumentKeywordLink (
	DocumentKey,
	KeywordKey
)
VALUES (
	@DocumentKey,
	@KeywordKey
)
GO

CREATE PROCEDURE [dbo].CreateDocumentKeywordLink (
	@DocumentKey int,
	@KeywordKey int
)
AS

IF NOT EXISTS (SELECT DocumentKey FROM DocumentKeywordLink WHERE DocumentKey = @DocumentKey AND KeywordKey = @KeywordKey)
	BEGIN
		INSERT INTO DocumentKeywordLink (
			DocumentKey,
			KeywordKey
		)
		VALUES (
			@DocumentKey,
			@KeywordKey
		)
	END
GO

CREATE PROCEDURE [dbo].DeleteDocumentKeywordLink (
	@DocumentKey int,
	@KeywordKey int
)
AS

DELETE FROM DocumentKeywordLink
WHERE DocumentKey = @DocumentKey AND KeywordKey = @KeywordKey

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].RetrieveDocumentKeywordLinkByKey (
	@DocumentKey int,
	@KeywordKey int
)
AS

SELECT
	DocumentKey,
	KeywordKey
FROM DocumentKeywordLink
WHERE DocumentKey = @DocumentKey AND KeywordKey = @KeywordKey

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].LoadDocumentKeywordLinkByDocumentKey (
	@DocumentKey int
)
AS

SELECT
	dkl.DocumentKey,
	dkl.KeywordKey
FROM DocumentKeywordLink dkl JOIN Keyword k ON dkl.KeywordKey = k.[Key]
WHERE dkl.DocumentKey = @DocumentKey
ORDER BY k.Word
GO

CREATE PROCEDURE [dbo].LoadDocumentKeywordLinkByKeywordKey (
	@KeywordKey int
)
AS

SELECT
	dkl.DocumentKey,
	dkl.KeywordKey
FROM DocumentKeywordLink dkl JOIN Document d ON dkl.DocumentKey = d.[Key]
WHERE dkl.KeywordKey = @KeywordKey
ORDER BY d.Title
GO