CREATE PROCEDURE [dbo].AddRepository (
	@UserKey int,
	@Name varchar(30),
	@Description varchar(255),
	@CreationDate datetime OUT
)
AS

SET @CreationDate = getdate()

INSERT INTO Repository (
	UserKey,
	CreationDate,
	[Name],
	Description
)
VALUES (
	@UserKey,
	@CreationDate,
	@Name,
	@Description
)

RETURN @@IDENTITY
GO

CREATE PROCEDURE [dbo].DeleteRepository (
	@Key int
)
AS

DELETE FROM Repository
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].RetrieveRepositoryByKey (
	@Key int
)
AS

SELECT
	[Key],
	UserKey,
	CreationDate,
	[Name],
	Description
FROM Repository
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].RetrieveRepositoryByName (
	@UserKey int,
	@Name varchar(30)
)
AS

SELECT
	[Key],
	UserKey,
	CreationDate,
	[Name],
	Description
FROM Repository
WHERE UserKey = @UserKey AND [Name] = @Name

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO

CREATE PROCEDURE [dbo].LoadRepository
AS

SELECT
	[Key],
	UserKey,
	CreationDate,
	[Name],
	Description
FROM Repository
ORDER BY UserKey
GO

CREATE PROCEDURE [dbo].LoadRepositoryByUserKey (
	@UserKey int
)
AS

SELECT
	[Key],
	UserKey,
	CreationDate,
	[Name],
	Description
FROM Repository
WHERE UserKey = @UserKey
ORDER BY UserKey
GO

CREATE PROCEDURE [dbo].UpdateRepository (
	@Key int,
	@UserKey int,
	@Name varchar(60),
	@Description varchar(128)
)
AS

UPDATE Repository
SET	UserKey = @UserKey,
	[Name] = @Name,
	Description = @Description
WHERE [Key] = @Key

IF @@ROWCOUNT = 0
	RETURN 0
ELSE
	RETURN 1
GO
