﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddDocument : PageBase
{
    protected string DocumentTitle
    {
        get
        {
            return mTitleTextBox.Text.Trim();
        }
        set
        {
            mTitleTextBox.Text = value.Trim();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        mRepository = GetSessionValue<RepositoryRecord>(SessionVariable.Repository);
        if (mRepository == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.ErrorPage) + "?ErrorMessage=Repository Not Set", true);
        }

        mStatusLabel.Visible = false;

        mReferringUrl = Request.Params["ReferringUrl"];
        mCancelButton.Visible = (string.IsNullOrEmpty(mReferringUrl) == false);

        PopulateAuthorDropDown();
        PopulatePublicationSourceDropDown();
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            mStatusLabel.Visible = false;
            DocumentRecord document = new DocumentRecord(SqlDataProvider.DefaultKey, mDatabase.DocumentTypes[DocumentType.AdobeAcrobat].Key, mRepository.Key, DocumentTitle, int.Parse(mAuthorDropDownList.SelectedValue), int.Parse(mPublicationSourceDropDownList.SelectedValue), (string.IsNullOrEmpty(mPublicationDateTextBox.Text) ? DateTime.Now.Date : DateTime.Parse(mPublicationDateTextBox.Text)), mAbstractTextBox.Text, mFileUpload.FileName, mFileUpload.FileBytes );
            DocumentManager.AddDocument(document);

            // Process keywords
            string keywordText = mKeywordsTextBox.Text.Trim();
            if (string.IsNullOrEmpty(keywordText) == false)
            {
                string[] keywords = keywordText.Split(KeywordDelimiter.ToCharArray());
                foreach (string keyword in keywords)
                {
                    if (string.IsNullOrEmpty(keyword) == false)
                    {
                        KeywordRecord keywordRecord = new KeywordRecord(SqlDataProvider.DefaultKey, keyword);
                        KeywordManager.CreateKeyword(keywordRecord);
                        DocumentKeywordLinkManager.CreateDocumentKeywordLink(new DocumentKeywordLinkRecord(document.Key, keywordRecord.Key));
                    }
                }
            }

            Response.Redirect(mReferringUrl, true);
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(mReferringUrl, true);
    }

    #endregion

    #region Private Methods

    private void PopulateAuthorDropDown()
    {
        AuthorRecordList authors = AuthorManager.LoadAuthor();
        foreach (AuthorRecord author in authors)
        {
            mAuthorDropDownList.Items.Add(new ListItem(author.FullName, author.Key.ToString()));
        }
    }

    private void PopulatePublicationSourceDropDown()
    {
        PublicationSourceRecordList publicationSources = PublicationSourceManager.LoadPublicationSource();
        foreach (PublicationSourceRecord publicationSource in publicationSources)
        {
            mPublicationSourceDropDownList.Items.Add(new ListItem(publicationSource.Name, publicationSource.Key.ToString()));
        }
    }

    #endregion

    private string KeywordDelimiter = " ";

    private UserRecord mUser;
    private RepositoryRecord mRepository;
    private string mReferringUrl;
}
