﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddGroup : PageBase
{
    protected string GroupName
    {
        get
        {
            return mNameTextBox.Text.Trim();
        }
        set
        {
            mNameTextBox.Text = value.Trim();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        mStatusLabel.Visible = false;
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            GroupRecord group;
            if (GroupManager.RetrieveGroupByName(mUser.Key, GroupName, out group))
            {
                mStatusLabel.Visible = true;
                mStatusLabel.Text = string.Format("Could not add group. Existing group named [{0}] found.", GroupName);
            }
            else
            {
                mStatusLabel.Visible = false;
                GroupManager.AddGroup(new GroupRecord(SqlDataProvider.DefaultKey, mUser.Key, GroupName));

                Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
            }
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
    }

    #endregion

    private UserRecord mUser;
}
