﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddGroupAdmin : PageBase
{
    protected string GroupName
    {
        get
        {
            return mNameTextBox.Text.Trim();
        }
        set
        {
            mNameTextBox.Text = value.Trim();
        }
    }

    protected int OwnerKey
    {
        get
        {
            return int.Parse(mUserDropDownList.SelectedValue);
        }
        set
        {
            mUserDropDownList.SelectedValue = value.ToString();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else if (mUser.IsSiteAdministrator == false)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        PopulateUserDropDown();

        mStatusLabel.Visible = false;
        mNameTextBox.Focus();
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            GroupRecord group;
            if (GroupManager.RetrieveGroupByName(OwnerKey, GroupName, out group))
            {
                mStatusLabel.Visible = true;
                mStatusLabel.Text = string.Format("Could not add group. Existing group [{0}] found.", GroupName);
            }
            else
            {
                mStatusLabel.Visible = false;
                GroupManager.AddGroup(new GroupRecord(SqlDataProvider.DefaultKey, OwnerKey, GroupName));

                Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Groups.ToString(), true);
            }
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Groups.ToString(), true);
    }

    #endregion

    #region Private Methods

    private void PopulateUserDropDown()
    {
        UserRecordList users = UserManager.LoadUser();
        foreach (UserRecord user in users)
        {
            mUserDropDownList.Items.Add(new ListItem(user.FullName, user.Key.ToString()));
        }
    }

    #endregion

    private UserRecord mUser;
}
