﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddRepository : PageBase
{
    protected string RepositoryName
    {
        get
        {
            return mNameTextBox.Text.Trim();
        }
        set
        {
            mNameTextBox.Text = value.Trim();
        }
    }

    protected string RepositoryDescription
    {
        get
        {
            return mDescriptionTextBox.Text.Trim();
        }
        set
        {
            mDescriptionTextBox.Text = value.Trim();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        mStatusLabel.Visible = false;
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            RepositoryRecord repository;
            if (RepositoryManager.RetrieveRepositoryByName(mUser.Key, RepositoryName, out repository))
            {
                mStatusLabel.Visible = true;
                mStatusLabel.Text = string.Format("Could not add repository. Existing repository named [{0}] found.", RepositoryName);
            }
            else
            {
                mStatusLabel.Visible = false;
                RepositoryManager.AddRepository(new RepositoryRecord(SqlDataProvider.DefaultKey, mUser.Key, default(DateTime), RepositoryName, RepositoryDescription));

                Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
            }
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
    }

    #endregion

    private UserRecord mUser;
}
