﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddRepositoryAdmin : PageBase
{
    protected string RepositoryName
    {
        get
        {
            return mNameTextBox.Text.Trim();
        }
        set
        {
            mNameTextBox.Text = value.Trim();
        }
    }

    protected int OwnerKey
    {
        get
        {
            return int.Parse(mUserDropDownList.SelectedValue);
        }
        set
        {
            mUserDropDownList.SelectedValue = value.ToString();
        }
    }

    protected string Description
    {
        get
        {
            return mDescriptionTextBox.Text.Trim();
        }
        set
        {
            mDescriptionTextBox.Text = value.Trim();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else if (mUser.IsSiteAdministrator == false)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        PopulateUserDropDown();

        mStatusLabel.Visible = false;
        mNameTextBox.Focus();
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            RepositoryRecord repository;
            if (RepositoryManager.RetrieveRepositoryByName(OwnerKey, RepositoryName, out repository))
            {
                mStatusLabel.Visible = true;
                mStatusLabel.Text = string.Format("Could not add repository. Existing repository [{0}] found.", RepositoryName);
            }
            else
            {
                mStatusLabel.Visible = false;
                RepositoryManager.AddRepository(new RepositoryRecord(SqlDataProvider.DefaultKey, OwnerKey, default(DateTime), RepositoryName, Description));

                Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Repositories.ToString(), true);
            }
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Repositories.ToString(), true);
    }

    #endregion

    #region Private Methods

    private void PopulateUserDropDown()
    {
        UserRecordList users = UserManager.LoadUser();
        foreach (UserRecord user in users)
        {
            mUserDropDownList.Items.Add(new ListItem(user.FullName, user.Key.ToString()));
        }
    }

    #endregion

    private UserRecord mUser;
}
