﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;
using Bespoke.Common.Data;

public partial class AddUserAdmin : PageBase
{
    protected string Username
    {
        get
        {
            return mUsernameTextBox.Text.Trim();
        }
        set
        {
            mUsernameTextBox.Text = value.Trim();
        }
    }

    protected string Password
    {
        get
        {
            return mPasswordTextBox.Text.Trim();
        }
        set
        {
            mPasswordTextBox.Text = value.Trim();
        }
    }

    protected bool IsSiteAministrator
    {
        get
        {
            return mIsSiteAdministratorCheckBox.Checked;
        }
        set
        {
            mIsSiteAdministratorCheckBox.Checked = value;
        }
    }

    protected string FullName
    {
        get
        {
            return mFullNameTextBox.Text.Trim();
        }
        set
        {
            mFullNameTextBox.Text = value.Trim();
        }
    }

    protected string EmailAddress
    {
        get
        {
            return mEmailAddressTextBox.Text.Trim();
        }
        set
        {
            mEmailAddressTextBox.Text = value.Trim();
        }
    }

    protected string URL
    {
        get
        {
            return mURLTextBox.Text.Trim();
        }
        set
        {
            mURLTextBox.Text = value.Trim();
        }
    }

    protected override void OnPageLoad()
    {
        mUser = GetSessionValue<UserRecord>(SessionVariable.User);
        if (mUser == null)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.LoginPage), true);
        }
        else if (mUser.IsSiteAdministrator == false)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.MainPage), true);
        }
        else
        {
            mLoginLabel.Text = mUser.Username + " logged in (<a href=\"" + GetApplicationValue<string>(ApplicationVariable.LogoutPage) + "\">Logout</a>)";
        }

        mStatusLabel.Visible = false;
        mUsernameTextBox.Focus();
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        try
        {
            UserRecord user;
            if (UserManager.RetrieveUserByUsername(Username, out user))
            {
                mStatusLabel.Visible = true;
                mStatusLabel.Text = string.Format("Could not add user. Existing username [{0}] found.", Username);
            }
            else
            {
                mStatusLabel.Visible = false;
                UserManager.AddUser(new UserRecord(SqlDataProvider.DefaultKey, Username, Password, IsSiteAministrator, FullName, EmailAddress, URL));

                Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Users.ToString(), true);
            }
        }
        catch (Exception ex)
        {
            mStatusLabel.Visible = true;
            mStatusLabel.Text = ex.Message;
        }
    }

    protected void mCancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Users.ToString(), true);
    }

    #endregion

    private UserRecord mUser;
}
