﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.SessionState;
using System.Collections.Generic;
using Bespoke.ResearchExpert.Data;

/// <summary>
/// Set of available application variables.
/// </summary>
public enum ApplicationVariable
{
    /// <summary>
    /// 
    /// </summary>
    MainPage = 0,

    /// <summary>
    /// 
    /// </summary>
    LoginPage,

    /// <summary>
    /// 
    /// </summary>
    LogoutPage,

    /// <summary>
    /// 
    /// </summary>
    SiteAdministrationPage,

    /// <summary>
    /// 
    /// </summary>
    ErrorPage
}

/// <summary>
/// Set of available session variables.
/// </summary>
public enum SessionVariable
{
    /// <summary>
    /// 
    /// </summary>
    User = 0,

    /// <summary>
    /// 
    /// </summary>
    Repository
}

/// <summary>
/// 
/// </summary>
public abstract class PageBase : System.Web.UI.Page
{
    public T GetApplicationValue<T>(ApplicationVariable name)
    {
        object value = Application[name.ToString()];

        if ((value != null) && (value is T == false))
        {
            throw new InvalidCastException("Application variable [" + name.ToString() + "] does not store type: " + typeof(T).ToString());
        }
        else
        {
            return (T)value;
        }
    }

    public T GetSessionValue<T>(SessionVariable name)
    {
        object value = Session[name.ToString()];

        if ((value != null) && (value is T == false))
        {
            throw new InvalidCastException("Session variable [" + name.ToString() + "] does not store type: " + typeof(T).ToString());
        }
        else
        {
            return (T)value;
        }
    }

    public void SaveSessionValue(SessionVariable name, object value)
    {
        Session[name.ToString()] = value;
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        mDatabase = new Database();
        mDatabase.Init(false);
        OnPageLoad();
    }

    protected void Page_Unload(object sender, EventArgs e)
    {
        if (mDatabase != null)
        {
            mDatabase.Shutdown();
        }

        OnPageUnload();
    }

    protected virtual void OnPageLoad()
    {
    }

    protected virtual void OnPageUnload()
    {
    }

    protected Control GetRootControl(Control control)
    {
        return (control.Parent == null ? control : GetRootControl(control.Parent));
    }

    protected Database mDatabase;
}