﻿using System;
using System.IO;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.Common;
using Bespoke.ResearchExpert.Data;

public partial class DocumentControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        RepositoryRecord repository = GetSessionValue<RepositoryRecord>(SessionVariable.Repository);
        if (repository != null)
        {
            DocumentRecordList documents = DocumentManager.LoadDocument(repository.Key);
            if (documents.Count == 0)
            {
                TableCell cell = new TableCell();
                cell.ColumnSpan = 3;
                cell.Text = "<< No Document Found >>";

                TableRow row = new TableRow();
                row.Cells.Add(cell);

                mDocumentTable.Rows.Add(row);
            }
            else
            {
                bool ToggleColor = false;

                foreach (DocumentRecord document in documents)
                {
                    // Inner table containing each document
                    Table innerTable = new Table();
                    innerTable.CssClass = "DocumentItem";

                    // First row -- title
                    TableRow innerRow = new TableRow();
                    innerTable.Rows.Add(innerRow);
                    TableCell innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem1";
                    innerCell.Text = "Title:";

                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem2";
                    innerCell.Text = document.Title;

                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem3";
                    DataButton documentButton = new DataButton();
                    innerCell.Controls.Add(documentButton);
                    documentButton.Text = "PDF";
                    documentButton.Tag = document;
                    documentButton.Click += new EventHandler(documentButton_Click);

                    // Second row -- authors
                    innerRow = new TableRow();
                    innerTable.Rows.Add(innerRow);
                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem1";
                    innerCell.Text = "Authors:";

                    //TODO: add multiple author support
                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem2";
                    innerCell.ColumnSpan = 2;
                    AuthorRecord author;
                    AuthorManager.RetrieveAuthorByKey(document.AuthorKey, out author);
                    HyperLink authorLink = new HyperLink();
                    innerCell.Controls.Add(authorLink);
                    authorLink.CssClass = "DocumentItem";
                    if (author != null)
                    {
                        authorLink.Text = author.FullName;
                        authorLink.NavigateUrl = Path.Combine(Request.ApplicationPath, string.Format("AuthorView.aspx?AuthorKey={0}", author.Key));
                    }
                    else
                    {
                        authorLink.Text = mDatabase.UnknownAuthor.FullName;
                        authorLink.NavigateUrl = Path.Combine(Request.ApplicationPath, string.Format("AuthorView.aspx?AuthorKey={0}", mDatabase.UnknownAuthor.Key));
                    }

                    // Third row -- publication source
                    innerRow = new TableRow();
                    innerTable.Rows.Add(innerRow);
                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem1";
                    innerCell.Text = "Source:";

                    innerCell = new TableCell();
                    innerRow.Cells.Add(innerCell);
                    innerCell.CssClass = "DocumentItem2";
                    innerCell.ColumnSpan = 2;
                    PublicationSourceRecord publicationSource;
                    PublicationSourceManager.RetrievePublicationSourceByKey(document.PublicationSourceKey, out publicationSource);
                    HyperLink publicationSourceLink = new HyperLink();
                    innerCell.Controls.Add(publicationSourceLink);
                    publicationSourceLink.CssClass = "DocumentItem";
                    if (publicationSource != null)
                    {
                        publicationSourceLink.Text = publicationSource.Name;
                        publicationSourceLink.NavigateUrl = Path.Combine(Request.ApplicationPath, string.Format("PublicationSourceView.aspx?PublicationSourceKey={0}", publicationSource.Key));
                    }
                    else
                    {
                        publicationSourceLink.Text = mDatabase.UnknownPublicationSource.Name;
                        publicationSourceLink.NavigateUrl = Path.Combine(Request.ApplicationPath, string.Format("PublicationSourceView.aspx?PublicationSourceKey={0}", mDatabase.UnknownPublicationSource.Key));
                    }

                    // Outer/containing table for background color toggling
                    ToggleColor = !ToggleColor;
                    string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                    TableRow row = new TableRow();
                    mDocumentTable.Rows.Add(row);
                    TableCell cell = new TableCell();
                    row.Cells.Add(cell);
                    cell.CssClass = cssClass;
                    cell.Controls.Add(innerTable);
                }
            }

            mAddDocumentLink.Visible = true;
            mAddDocumentLink.NavigateUrl = Path.Combine(Request.ApplicationPath, "AddDocument.aspx?ReferringUrl=DocumentView.aspx");
        }
        else
        {
            mAddDocumentLink.Visible = false;
        }
    }

    /// <summary>
    /// Handles the Click event of the documentButton control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void documentButton_Click(object sender, EventArgs e)
    {
        try
        {
            DocumentRecord document = (DocumentRecord)((DataButton)sender).Tag;
            DocumentTypeRecord documentType;
            Assert.IsTrue("documentType", DocumentTypeManager.RetrieveDocumentTypeByKey(document.DocumentTypeKey, out documentType));

            Response.ContentType = documentType.HttpContentType;
            Response.AddHeader("Content-Disposition", string.Format("attachment; filename={0}", document.FileName));
            Response.BinaryWrite(document.Data);
        }
        catch (Exception ex)
        {
            Response.Redirect(GetApplicationValue<string>(ApplicationVariable.ErrorPage) + "?ErrorMessage=" + ex.Message, true);
        }
    }

    #region Event Handlers

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    protected void mFilterButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    #endregion

    private const string CssCellStyle = "DocumentControl";
    private const string CssCellStyleAlt = "DocumentControlAlt";
}
