﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class GroupAdminControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        GroupRecordList groups = GroupManager.LoadGroup();
        if (groups.Count == 0)
        {
            TableCell cell = new TableCell();
            cell.ColumnSpan = 3;
            cell.Text = "<< No Groups Found >>";

            TableRow row = new TableRow();
            row.Cells.Add(cell);
            mGroupTable.Rows.Add(row);

            mDeleteButton.Enabled = false;
        }
        else
        {
            mDeleteButton.Enabled = true;
            mDeleteButton.Attributes.Add("onclick", "return confirm('Are you sure you want to delete these groups?');");
            
            bool ToggleColor = false;

            foreach (GroupRecord group in groups)
            {
                ToggleColor = !ToggleColor;
                string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                TableRow row = new TableRow();
                mGroupTable.Rows.Add(row);

                TableCell cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                DataCheckBox selectCheckBox = new DataCheckBox();
                selectCheckBox.Tag = group;
                cell.Controls.Add(selectCheckBox);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = string.Format("<a class=\"GroupAdminItem\" href=\"EditGroup.aspx?GroupKey={0}\">{1}</a>", group.Key.ToString(), group.Name);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                UserRecord user;
                UserManager.RetrieveUserByKey(group.UserKey, out user);
                if (user != null)
                {
                    cell.Text = user.Username;
                }
            }
        }
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(Path.Combine(Request.ApplicationPath, "AddGroupAdmin.aspx"), true);
    }

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        foreach (Control control in mGroupTable.Controls)
        {
            DeleteGroupInChildControl(control);
        }

        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Groups.ToString(), true);
    }

    protected void mFilterButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    protected void mSelectAllCheckBox_CheckedChanged(object sender, EventArgs e)
    {
        foreach (Control control in mGroupTable.Controls)
        {
            CheckChildControls(control, mSelectAllCheckBox.Checked);
        }
    }

    #endregion

    private void DeleteGroupInChildControl(Control parentControl)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            DeleteGroupInChildControl(childControl);
        }

        DataCheckBox checkBox = parentControl as DataCheckBox;
        if (checkBox != null && checkBox.Checked)
        {
            GroupRecord group = checkBox.Tag as GroupRecord;
            if (group != null)
            {
                GroupManager.DeleteGroup(group);
            }
        }
    }

    private void CheckChildControls(Control parentControl, bool isChecked)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            CheckChildControls(childControl, isChecked);
        }

        CheckBox checkBox = parentControl as CheckBox;
        if (checkBox != null)
        {
            checkBox.Checked = isChecked;
        }
    }

    private const string CssCellStyle = "GroupAdminControl";
    private const string CssCellStyleAlt = "GroupAdminControlAlt";
}
