﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class LoginControl : ControlBase
{
    #region Events

    /// <summary>
    /// Occurs when [logging in].
    /// </summary>
    public event EventHandler LoggingIn;

    #endregion

    #region Properties

    public Panel LoginPanel
    {
        get
        {
            return mLoginPanel;
        }
    }

    /// <summary>
    /// Gets or sets the username.
    /// </summary>
    /// <value>The username.</value>
    public string Username
    {
        get
        {
            return mUsername.Text;
        }
        set
        {
            mUsername.Text = value;
        }
    }

    /// <summary>
    /// Gets or sets the password.
    /// </summary>
    /// <value>The password.</value>
    public string Password
    {
        get
        {
            return mPassword.Text;
        }
        set
        {
            mPassword.Text = value;
        }
    }

    /// <summary>
    /// Gets or sets the status label.
    /// </summary>
    /// <value>The status label.</value>
    public string StatusLabel
    {
        get
        {
            return mStatusLabel.Text;
        }
        set
        {
            mStatusLabel.Text = value;
        }
    }

    #endregion

    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        mUsername.Focus();
    }

    /// <summary>
    /// Handles the Click event of the mLoginButton control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void mLoginButton_Click(object sender, EventArgs e)
    {
        OnLoggingIn();
    }

    /// <summary>
    /// Called when [logging in].
    /// </summary>
    protected void OnLoggingIn()
    {
        if (LoggingIn != null)
        {
            LoggingIn(this, EventArgs.Empty);
        }
    }
}
