﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class PublicationSourceTypeAdminControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        PublicationSourceTypeRecordList publicationSourceTypes = PublicationSourceTypeManager.LoadPublicationSourceType();
        if (publicationSourceTypes.Count == 0)
        {
            TableCell cell = new TableCell();
            cell.ColumnSpan = 3;
            cell.Text = "<< No Publication Source Types Found >>";

            TableRow row = new TableRow();
            row.Cells.Add(cell);
            mGroupTable.Rows.Add(row);

            mDeleteButton.Enabled = false;
        }
        else
        {
            mDeleteButton.Enabled = true;
            mDeleteButton.Attributes.Add("onclick", "return confirm('Are you sure you want to delete these publication source types?');");
            
            bool ToggleColor = false;

            foreach (PublicationSourceTypeRecord publicationSourceType in publicationSourceTypes)
            {
                ToggleColor = !ToggleColor;
                string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                TableRow row = new TableRow();
                mGroupTable.Rows.Add(row);

                TableCell cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                DataCheckBox selectCheckBox = new DataCheckBox();
                selectCheckBox.Tag = publicationSourceType;
                cell.Controls.Add(selectCheckBox);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = string.Format("<a class=\"PublicationSourceTypeAdminItem\" href=\"EditPublicationSourceType.aspx?PublicationSourceTypeKey={0}\">{1}</a>", publicationSourceType.Key.ToString(), publicationSourceType.Name);
            }
        }
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(Path.Combine(Request.ApplicationPath, "AddPublicationSourceTypeAdmin.aspx"), true);
    }

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        foreach (Control control in mGroupTable.Controls)
        {
            DeletePublicationSourceTypeInChildControl(control);
        }

        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.PublicationSourceTypes.ToString(), true);
    }

    protected void mSelectAllCheckBox_CheckedChanged(object sender, EventArgs e)
    {
        foreach (Control control in mGroupTable.Controls)
        {
            CheckChildControls(control, mSelectAllCheckBox.Checked);
        }
    }

    #endregion

    private void DeletePublicationSourceTypeInChildControl(Control parentControl)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            DeletePublicationSourceTypeInChildControl(childControl);
        }

        DataCheckBox checkBox = parentControl as DataCheckBox;
        if (checkBox != null && checkBox.Checked)
        {
            PublicationSourceTypeRecord publicationSourceType = checkBox.Tag as PublicationSourceTypeRecord;
            if (publicationSourceType != null)
            {
                PublicationSourceTypeManager.DeletePublicationSourceType(publicationSourceType);
            }
        }
    }

    private void CheckChildControls(Control parentControl, bool isChecked)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            CheckChildControls(childControl, isChecked);
        }

        CheckBox checkBox = parentControl as CheckBox;
        if (checkBox != null)
        {
            checkBox.Checked = isChecked;
        }
    }

    private const string CssCellStyle = "PublicationSourceTypeAdminControl";
    private const string CssCellStyleAlt = "PublicationSourceTypeAdminControlAlt";
}
