﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class RepositoryAdminControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        RepositoryRecordList repositories = RepositoryManager.LoadRepository();
        if (repositories.Count == 0)
        {
            TableCell cell = new TableCell();
            cell.ColumnSpan = 3;
            cell.Text = "<< No Repositories Found >>";

            TableRow row = new TableRow();
            row.Cells.Add(cell);
            mRepositoryTable.Rows.Add(row);

            mDeleteButton.Enabled = false;
        }
        else
        {
            mDeleteButton.Enabled = true;
            mDeleteButton.Attributes.Add("onclick", "return confirm('Are you sure you want to delete these repositories?');");
            
            bool ToggleColor = false;

            foreach (RepositoryRecord repository in repositories)
            {
                ToggleColor = !ToggleColor;
                string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                TableRow row = new TableRow();
                mRepositoryTable.Rows.Add(row);

                TableCell cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                DataCheckBox selectCheckBox = new DataCheckBox();
                selectCheckBox.Tag = repository;
                cell.Controls.Add(selectCheckBox);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = string.Format("<a class=\"RepositoryAdminItem\" href=\"EditRepository.aspx?RepositoryKey={0}\">{1}</a>", repository.Key.ToString(), repository.Name);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                UserRecord user;
                UserManager.RetrieveUserByKey(repository.UserKey, out user);
                if (user != null)
                {
                    cell.Text = user.Username;
                }

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = repository.CreationDate.ToShortDateString();
            }
        }
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(Path.Combine(Request.ApplicationPath, "AddRepositoryAdmin.aspx"), true);
    }

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        foreach (Control control in mRepositoryTable.Controls)
        {
            DeleteRepositoryInChildControl(control);
        }

        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Repositories.ToString(), true);
    }

    protected void mFilterButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    protected void mSelectAllCheckBox_CheckedChanged(object sender, EventArgs e)
    {
        foreach (Control control in mRepositoryTable.Controls)
        {
            CheckChildControls(control, mSelectAllCheckBox.Checked);
        }
    }

    #endregion

    private void DeleteRepositoryInChildControl(Control parentControl)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            DeleteRepositoryInChildControl(childControl);
        }

        DataCheckBox checkBox = parentControl as DataCheckBox;
        if (checkBox != null && checkBox.Checked)
        {
            RepositoryRecord repository = checkBox.Tag as RepositoryRecord;
            if (repository != null)
            {
                RepositoryManager.DeleteRepository(repository);
            }
        }
    }

    private void CheckChildControls(Control parentControl, bool isChecked)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            CheckChildControls(childControl, isChecked);
        }

        CheckBox checkBox = parentControl as CheckBox;
        if (checkBox != null)
        {
            checkBox.Checked = isChecked;
        }
    }

    private const string CssCellStyle = "RepositoryAdminControl";
    private const string CssCellStyleAlt = "RepositoryAdminControlAlt";
}
