﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class RepositoryControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        UserRecord user = GetSessionValue<UserRecord>(SessionVariable.User);
        if (user != null)
        {
            RepositoryRecordList repositories = RepositoryManager.LoadRepositoryByUserKey(user.Key);

            if (repositories.Count == 0)
            {
                TableCell cell = new TableCell();
                cell.ColumnSpan = 3;
                cell.Text = "<< No Repositories Found >>";

                TableRow row = new TableRow();
                row.Cells.Add(cell);

                mRepositoryTable.Rows.Add(row);
            }
            else
            {
                bool ToggleColor = false;

                foreach (RepositoryRecord repository in repositories)
                {
                    ToggleColor = !ToggleColor;
                    string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                    TableRow row = new TableRow();                    

                    TableCell cell = new TableCell();
                    cell.CssClass = cssClass;
                    cell.Text = string.Format("<a class=\"RepositoryItem\" href=\"DocumentView.aspx?RepositoryKey={0}\">{1}</a>", repository.Key.ToString(), repository.Name);
                    row.Cells.Add(cell);

                    cell = new TableCell();
                    //TODO
                    cell.CssClass = cssClass;
                    cell.Text = (repository.UserKey == user.Key ? "Owner" : "Member");
                    row.Cells.Add(cell);

                    int documentCount = DocumentManager.GetDocumentCount(repository.Key);
                    cell = new TableCell();
                    cell.CssClass = cssClass;
                    cell.Text = documentCount.ToString();
                    row.Cells.Add(cell);

                    mRepositoryTable.Rows.Add(row);
                }
            }
        }
    }

    #region Event Handlers

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    protected void mFilterButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    #endregion

    private const string CssCellStyle = "RepositoryControl";
    private const string CssCellStyleAlt = "RepositoryControlAlt";

}
