﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bespoke.ResearchExpert.Data;

public partial class UserAdminControl : ControlBase
{
    /// <summary>
    /// Called when [page load].
    /// </summary>
    protected override void OnPageLoad()
    {
        UserRecordList users = UserManager.LoadUser();
        if (users.Count == 0)
        {
            TableCell cell = new TableCell();
            cell.ColumnSpan = 3;
            cell.Text = "<< No Users Found >>";

            TableRow row = new TableRow();
            row.Cells.Add(cell);
            mUserTable.Rows.Add(row);

            mDeleteButton.Enabled = false;
        }
        else
        {
            mDeleteButton.Enabled = true;
            mDeleteButton.Attributes.Add("onclick", "return confirm('Are you sure you want to delete these users?');");
            
            bool ToggleColor = false;

            foreach (UserRecord user in users)
            {
                ToggleColor = !ToggleColor;
                string cssClass = (ToggleColor ? CssCellStyle : CssCellStyleAlt);
                TableRow row = new TableRow();
                mUserTable.Rows.Add(row);

                TableCell cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                DataCheckBox selectCheckBox = new DataCheckBox();
                selectCheckBox.Tag = user;
                cell.Controls.Add(selectCheckBox);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = string.Format("<a class=\"UserAdminItem\" href=\"EditUser.aspx?UserKey={0}\">{1}</a>", user.Key.ToString(), user.FullName);

                cell = new TableCell();
                row.Cells.Add(cell);
                cell.CssClass = cssClass;
                cell.Text = user.Username;
            }
        }
    }

    #region Event Handlers

    protected void mAddButton_Click(object sender, EventArgs e)
    {
        Response.Redirect(Path.Combine(Request.ApplicationPath, "AddUserAdmin.aspx"), true);
    }

    protected void mDeleteButton_Click(object sender, EventArgs e)
    {
        foreach (Control control in mUserTable.Controls)
        {
            DeleteUserInChildControl(control);
        }

        Response.Redirect(GetApplicationValue<string>(ApplicationVariable.SiteAdministrationPage) + "?SelectedTab=" + SiteAdministrationTabs.Users.ToString(), true);
    }

    protected void mFilterButton_Click(object sender, EventArgs e)
    {
        //TODO
    }

    protected void mSelectAllCheckBox_CheckedChanged(object sender, EventArgs e)
    {
        foreach (Control control in mUserTable.Controls)
        {
            CheckChildControls(control, mSelectAllCheckBox.Checked);
        }
    }

    #endregion

    #region Private Methods

    private void DeleteUserInChildControl(Control parentControl)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            DeleteUserInChildControl(childControl);
        }

        DataCheckBox checkBox = parentControl as DataCheckBox;
        if (checkBox != null && checkBox.Checked)
        {
            UserRecord user = checkBox.Tag as UserRecord;
            if (user != null)
            {
                UserManager.DeleteUser(user);
            }
        }
    }

    private void CheckChildControls(Control parentControl, bool isChecked)
    {
        foreach (Control childControl in parentControl.Controls)
        {
            CheckChildControls(childControl, isChecked);
        }

        CheckBox checkBox = parentControl as CheckBox;
        if (checkBox != null)
        {
            checkBox.Checked = isChecked;
        }
    }

    #endregion

    private const string CssCellStyle = "UserAdminControl";
    private const string CssCellStyleAlt = "UserAdminControlAlt";
}
